searchData={"items":[{"type":"module","title":"LoggerJSON","doc":"A collection of formatters and utilities for JSON-based logging for various cloud tools and platforms.","ref":"LoggerJSON.html"},{"type":"module","title":"Supported formatters - LoggerJSON","doc":"* `LoggerJSON.Formatters.Basic` - a basic JSON formatter that logs messages in a structured format,\ncan be used with any JSON-based logging system, like ElasticSearch, Logstash, etc.\n\n* `LoggerJSON.Formatters.GoogleCloud` - a formatter that logs messages in a structured format that can be\nconsumed by Google Cloud Logger and Google Cloud Error Reporter.\n\n* `LoggerJSON.Formatters.Datadog` - a formatter that logs messages in a structured format that can be consumed\nby Datadog.\n\n* `LoggerJSON.Formatters.Elastic` - a formatter that logs messages in a structured format that conforms to the\n[Elastic Common Schema (ECS)](https://www.elastic.co/guide/en/ecs/8.11/ecs-reference.html),\nso it can be consumed by ElasticSearch, LogStash, FileBeat and Kibana.","ref":"LoggerJSON.html#module-supported-formatters"},{"type":"module","title":"Installation - LoggerJSON","doc":"Add `logger_json` to your list of dependencies in `mix.exs`:\n\n    def deps do\n      [\n        # ...\n        {:logger_json, \"~> 7.0\"}\n        # ...\n      ]\n    end\n\nand install it running `mix deps.get`.\n\nThen, enable the formatter in your `runtime.exs`:\n\n    config :logger, :default_handler,\n      formatter: LoggerJSON.Formatters.Basic.new(metadata: [:request_id])\n\nor inside your application code (eg. in your `application.ex`):\n\n    formatter = LoggerJSON.Formatters.Basic.new(metadata: :all)\n    :logger.update_handler_config(:default, :formatter, formatter)\n\nor inside your `config.exs` (notice that `new/1` is not available here\nand tuple format must be used):\n\n    config :logger, :default_handler,\n      formatter: {LoggerJSON.Formatters.Basic, metadata: [:request_id]}","ref":"LoggerJSON.html#module-installation"},{"type":"module","title":"Configuration - LoggerJSON","doc":"Configuration can be set using `new/1` helper of the formatter module,\nor by setting the 2nd element of the `:formatter` option tuple in `Logger` configuration.\n\nFor example in `config.exs`:\n\n    config :logger, :default_handler,\n      formatter: LoggerJSON.Formatters.GoogleCloud.new(metadata: :all, project_id: \"logger-101\")\n\nor during runtime:\n\n    formatter = LoggerJSON.Formatters.Basic.new(metadata: {:all_except, [:conn]})\n    :logger.update_handler_config(:default, :formatter, formatter)\n\nBy default, `LoggerJSON` is using `Jason` as the JSON encoder. If you use Elixir 1.18 or later, you can\nuse the built-in `JSON` module as the encoder. To do this, you need to set the `:encoder` option in your\n`config.exs` file. This setting is only available at compile-time:\n\n    config :logger_json, encoder: JSON","ref":"LoggerJSON.html#module-configuration"},{"type":"module","title":"Shared Options - LoggerJSON","doc":"Some formatters require additional configuration options. Here are the options that are common for each formatter:\n\n  * `:encoder_opts` - options to be passed directly to the JSON encoder. This allows you to customize the behavior\n  of the JSON encoder. If the encoder is `JSON`, it defaults to `JSON.protocol_encode/2`. Otherwise, defaults to\n  empty keywords. See the [documentation for Jason](https://hexdocs.pm/jason/Jason.html#encode/2-options) for\n  available options for `Jason` encoder.\n\n  * `:metadata` - a list of metadata keys to include in the log entry. By default, no metadata is included.\n  If `:all`is given, all metadata is included. If `{:all_except, keys}` is given, all metadata except\n  the specified keys is included. If `{:from_application_env, {app, module}, path}` is given, the metadata is fetched from\n  the application environment (eg. `{:from_application_env, {:logger, :default_formatter}, [:metadata]}`) during the\n  configuration initialization.\n\n  * `:redactors` - a list of tuples, where first element is the module that implements the `LoggerJSON.Redactor` behaviour,\n  and the second element is the options to pass to the redactor module. By default, no redactors are used.","ref":"LoggerJSON.html#module-shared-options"},{"type":"module","title":"Metadata - LoggerJSON","doc":"You can set some well-known metadata keys to be included in the log entry. The following keys are supported\nfor all formatters:\n\n  * `:conn` - the `Plug.Conn` struct, setting it will include the request and response details in the log entry;\n  * `:crash_reason` - a tuple where the first element is the exception struct and the second is the stacktrace.\n  For example: `Logger.error(\"Exception!\", crash_reason: {e, __STACKTRACE__})`. Setting it will include the exception\n  details in the log entry.\n\nFormatters may encode the well-known metadata differently and support additional metadata keys, see the documentation\nof the formatter for more details.","ref":"LoggerJSON.html#module-metadata"},{"type":"function","title":"LoggerJSON.configure_log_level!/1","doc":"Changes Logger log level at runtime.\n\nNotice that settings this value below `compile_time_purge_level` would not work,\nbecause Logger calls would be already stripped at compile-time.","ref":"LoggerJSON.html#configure_log_level!/1"},{"type":"function","title":"LoggerJSON.configure_log_level_from_env!/1","doc":"Configures Logger log level at runtime by using value from environment variable.\n\nBy default, 'LOG_LEVEL' environment variable is used.","ref":"LoggerJSON.html#configure_log_level_from_env!/1"},{"type":"module","title":"LoggerJSON.Ecto","doc":"A telemetry handler that logs Ecto query metrics in JSON format.\n\nPlease keep in mind that logging all database operations will have a performance impact\non your application, it's not recommended to use this module in high-throughput production\nenvironments.","ref":"LoggerJSON.Ecto.html"},{"type":"function","title":"LoggerJSON.Ecto.attach/3","doc":"Attaches the telemetry handler to the given event.","ref":"LoggerJSON.Ecto.html#attach/3"},{"type":"function","title":"Available options - LoggerJSON.Ecto.attach/3","doc":"* `:level` - log level which is used to log requests. Defaults to `:info`.","ref":"LoggerJSON.Ecto.html#attach/3-available-options"},{"type":"function","title":"Dynamic log level - LoggerJSON.Ecto.attach/3","doc":"In some cases you may wish to set the log level dynamically\non a per-query basis. To do so, set the `:level` option to\na tuple, `{Mod, Fun, Args}`. The query and map of time measures\nwill be prepended to the provided list of arguments.\n\nWhen invoked, your function must return a\n[`Logger.level()`](`t:Logger.level()/0`) or `false` to\ndisable logging for the request.","ref":"LoggerJSON.Ecto.html#attach/3-dynamic-log-level"},{"type":"function","title":"Examples - LoggerJSON.Ecto.attach/3","doc":"Attaching the telemetry handler to the `MyApp.Repo` events with the `:info` log level:\n\n    LoggerJSON.Ecto.attach(\"logger-json-queries\", [:my_app, :repo, :query], :info)\n\nFor more details on event and handler naming see\n(`Ecto.Repo` documentation)[https://hexdocs.pm/ecto/Ecto.Repo.html#module-telemetry-events].","ref":"LoggerJSON.Ecto.html#attach/3-examples"},{"type":"function","title":"LoggerJSON.Ecto.telemetry_logging_handler/4","doc":"A telemetry handler that logs Ecto query along with it's metrics in a structured format.","ref":"LoggerJSON.Ecto.html#telemetry_logging_handler/4"},{"type":"behaviour","title":"LoggerJSON.Formatter","doc":"","ref":"LoggerJSON.Formatter.html"},{"type":"type","title":"LoggerJSON.Formatter.config/0","doc":"","ref":"LoggerJSON.Formatter.html#t:config/0"},{"type":"type","title":"LoggerJSON.Formatter.encoder_opts/0","doc":"","ref":"LoggerJSON.Formatter.html#t:encoder_opts/0"},{"type":"callback","title":"LoggerJSON.Formatter.format/2","doc":"Formats a log event.","ref":"LoggerJSON.Formatter.html#c:format/2"},{"type":"callback","title":"LoggerJSON.Formatter.new/1","doc":"Initializes a new formatter configuration.","ref":"LoggerJSON.Formatter.html#c:new/1"},{"type":"callback","title":"Compile‑time vs. Runtime Configuration - LoggerJSON.Formatter.new/1","doc":"This function can’t be used in `config.exs` because that file is evaluated\nbefore your application modules are compiled and loaded, so `new/1` isn’t defined yet.\nYou can only call it in `config/runtime.exs` or from your application code.\n\nIf you must set up the formatter in `config.exs`, use the tuple format:\nthe first element is the module implementing `LoggerJSON.Formatter`,\nand the second is the options passed to `new/1`. For example:\n\n  config :logger, :default_handler,\n    formatter: {LoggerJSON.Formatters.Basic, metadata: [:request_id]}\n\nNote that tuple‑based configs are resolved for each log entry,\nwhich can increase logging overhead.","ref":"LoggerJSON.Formatter.html#c:new/1-compile-time-vs-runtime-configuration"},{"type":"type","title":"LoggerJSON.Formatter.opts/0","doc":"","ref":"LoggerJSON.Formatter.html#t:opts/0"},{"type":"module","title":"LoggerJSON.Formatter.RedactorEncoder","doc":"","ref":"LoggerJSON.Formatter.RedactorEncoder.html"},{"type":"function","title":"LoggerJSON.Formatter.RedactorEncoder.encode/2","doc":"Takes a term and makes sure that it can be encoded by the encoder without errors\nand without leaking sensitive information.","ref":"LoggerJSON.Formatter.RedactorEncoder.html#encode/2"},{"type":"function","title":"Encoding rules - LoggerJSON.Formatter.RedactorEncoder.encode/2","doc":"Type                | Encoding                                            | Redaction\n------------------- | --------------------------------------------------- | --------------\n`boolean()`         | unchanged                                           | unchanged\n`map()`             | unchanged                                           | values are redacted\n`list()`            | unchanged                                           | unchanged\n`tuple()`           | converted to list                                   | unchanged\n`binary()`          | unchanged if printable, otherwise using `inspect/2` | unchanged\n`number()`          | unchanged                                           | unchanged\n`atom()`            | unchanged                                           | unchanged\n`struct()`          | converted to map                                    | values are redacted\n`keyword()`         | converted to map                                    | values are redacted\n`%Jason.Fragment{}` | unchanged                                           | unchanged if encoder is `Jason`\neverything else     | using `inspect/2`                                   | unchanged","ref":"LoggerJSON.Formatter.RedactorEncoder.html#encode/2-encoding-rules"},{"type":"type","title":"LoggerJSON.Formatter.RedactorEncoder.redactor/0","doc":"","ref":"LoggerJSON.Formatter.RedactorEncoder.html#t:redactor/0"},{"type":"module","title":"LoggerJSON.Formatters.Basic","doc":"Custom Erlang's [`:logger` formatter](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#formatters) which\nwrites logs in a JSON format.\n\nFor list of options see \"Shared options\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.Basic.html"},{"type":"module","title":"Examples - LoggerJSON.Formatters.Basic","doc":"%{\n      \"message\" => \"Hello\",\n      \"metadata\" => %{\"domain\" => [\"elixir\"]},\n      \"severity\" => \"notice\",\n      \"time\" => \"2024-04-11T21:31:01.403Z\"\n    }","ref":"LoggerJSON.Formatters.Basic.html#module-examples"},{"type":"module","title":"LoggerJSON.Formatters.Datadog","doc":"Custom Erlang's [`:logger` formatter](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#formatters) which\nwrites logs in a structured format that can be consumed by Datadog.\n\nThis formatter adheres to the\n[default standard attribute list](https://docs.datadoghq.com/logs/processing/attributes_naming_convention/#default-standard-attribute-list)\nas much as possible.","ref":"LoggerJSON.Formatters.Datadog.html"},{"type":"module","title":"Formatter Configuration - LoggerJSON.Formatters.Datadog","doc":"The formatter can be configured with the following options:\n\n* `:hostname` (optional) - changes how the `syslog.hostname` is set in logs. By default, it uses `:system` which uses\n  `:inet.gethostname/0` to resolve the value. If you are running in an environment where the hostname is not correct,\n  you can hard code it by setting `hostname` to a string. In places where the hostname is inaccurate but also dynamic\n  (like Kubernetes), you can set `hostname` to `:unset` to exclude it entirely. You'll then be relying on\n  [`dd-agent`](https://docs.datadoghq.com/agent/) to determine the hostname.\n\n* `:reported_levels` (optional) - a list of log levels that should be reported as errors to Datadog.\n  Default: `[:emergency, :alert, :critical, :error]`.\n\nFor list of shared options see \"Shared options\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.Datadog.html#module-formatter-configuration"},{"type":"module","title":"Metadata - LoggerJSON.Formatters.Datadog","doc":"For list of other well-known metadata keys see \"Metadata\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.Datadog.html#module-metadata"},{"type":"module","title":"Examples - LoggerJSON.Formatters.Datadog","doc":"%{\n      \"domain\" => [\"elixir\"],\n      \"logger\" => %{\n        \"file_name\" => \"/Users/andrew/Projects/os/logger_json/test/formatters/datadog_test.exs\",\n        \"line\" => 44,\n        \"method_name\" => \"Elixir.LoggerJSON.Formatters.DatadogTest.test logs an LogEntry of a given level/1\",\n        \"thread_name\" => \"#PID<0.234.0>\"\n      },\n      \"message\" => \"Hello\",\n      \"syslog\" => %{\n        \"hostname\" => \"MacBook-Pro\",\n        \"severity\" => \"notice\",\n        \"timestamp\" => \"2024-04-11T23:03:39.726Z\"\n      }\n    }","ref":"LoggerJSON.Formatters.Datadog.html#module-examples"},{"type":"module","title":"LoggerJSON.Formatters.Elastic","doc":"Custom Erlang's [`:logger` formatter](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#formatters) which\nwrites logs in a JSON-structured format that conforms to the Elastic Common Schema (ECS), so it can be consumed by\nElasticSearch, LogStash, FileBeat and Kibana.","ref":"LoggerJSON.Formatters.Elastic.html"},{"type":"module","title":"Formatter Configuration - LoggerJSON.Formatters.Elastic","doc":"For list of options see \"Shared options\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.Elastic.html#module-formatter-configuration"},{"type":"module","title":"Metadata - LoggerJSON.Formatters.Elastic","doc":"For list of other well-known metadata keys see \"Metadata\" in `LoggerJSON`.\n\nAny custom metadata that you set with `Logger.metadata/1` will be included top-level in the log entry.","ref":"LoggerJSON.Formatters.Elastic.html#module-metadata"},{"type":"module","title":"Examples - LoggerJSON.Formatters.Elastic","doc":"Example of an info log (`Logger.info(\"Hello\")` without any metadata):\n\n```elixir\n%{\n  \"@timestamp\" => \"2024-05-17T16:20:00.000Z\",\n  \"ecs.version\" => \"8.11.0\",\n  \"log.level\" => \"info\",\n  \"log.logger\" => \"Elixir.LoggerJSON.Formatters.ElasticTest\",\n  \"log.origin\" => %{\n    \"file.name\" => \"/app/logger_json/test/formatters/elastic_test.exs\",\n    \"file.line\" => 18,\n    \"function\" => \"test logs an LogEntry of every level/1\"\n  },\n  \"message\" => \"Hello\"\n}\n```\n\nExample of logging by keywords or by map (Logger.info(%{message: \"Hello\", foo: :bar, fiz: %{buz: \"buz\"}})).\nThe keywords or map items are added to the top-level of the log entry:\n\n```elixir\n%{\n  \"@timestamp\" => \"2024-05-17T16:20:00.000Z\",\n  \"ecs.version\" => \"8.11.0\",\n  \"fiz\" => %{\"buz\" => \"buz\"},\n  \"foo\" => \"bar\",\n  \"log.level\" => \"debug\",\n  \"log.logger\" => \"Elixir.LoggerJSON.Formatters.ElasticTest\",\n  \"log.origin\" => %{\n    \"file.line\" => 68,\n    \"file.name\" => \"/app/logger_json/test/formatters/elastic_test.exs\",\n    \"function\" => \"test logs an LogEntry with a map payload containing message/1\"\n  },\n  \"message\" => \"Hello\"\n}\n```\n\nExample of logging due to raising an exception (`raise RuntimeError`):\n\n```elixir\n%{\n  \"@timestamp\" => \"2024-05-17T16:20:00.000Z\",\n  \"ecs.version\" => \"8.11.0\",\n  \"error.message\" => \"runtime error\",\n  \"error.stack_trace\" => \"** (RuntimeError) runtime error\n    test/logger_json/formatters/elastic_test.exs:191: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\"test logs exceptions\"/1\n\",\n  \"error.type\" => \"Elixir.RuntimeError\",\n  \"log.level\" => \"error\",\n  \"message\" => \"Process #PID<0.322.0> raised an exception\n** (RuntimeError) runtime error\n    test/logger_json/formatters/elastic_test.exs:191: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\"test logs exceptions\"/1\"\n}\n```\n\nNote that if you raise an exception that contains an `id` or a `code` property, they will be included in the log entry as `error.id` and `error.code` respectively.\n\nExample:\n\n```elixir\ndefmodule TestException do\n  defexception [:message, :id, :code]\nend\n\n...\n\nraise TestException, id: :oops_id, code: 42, message: \"oops!\"\n```\n\nresults in:\n\n```elixir\n%{\n  \"@timestamp\" => \"2024-05-17T16:20:00.000Z\",\n  \"ecs.version\" => \"8.11.0\",\n  \"error.code\" => 42,\n  \"error.id\" => \"oops_id\",\n  \"error.message\" => \"oops!\",\n  \"error.stack_trace\" => \"** (LoggerJSON.Formatters.ElasticTest.TestException) oops!\n    test/logger_json/formatters/elastic_test.exs:223: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\"test logs exceptions with id and code\"/1\n\",\n  \"error.type\" => \"Elixir.LoggerJSON.Formatters.ElasticTest.TestException\",\n  \"log.level\" => \"error\",\n  \"message\" => \"Process #PID<0.325.0> raised an exception\n** (LoggerJSON.Formatters.ElasticTest.TestException) oops!\n    test/logger_json/formatters/elastic_test.exs:223: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\"test logs exceptions with id and code\"/1\"\n}\n```\n\nYou can also choose to log caught exceptions with a custom message.\nFor example, after catching an exception with `try`/`rescue`:\n\n```elixir\ntry do\n  raise \"oops\"\nrescue\n  e in RuntimeError -> Logger.error(\"Something went wrong\", crash_reason: {e, __STACKTRACE__})\nend\n```\n\nthen you'll get a message like:\n\n```elixir\n%{\n  \"@timestamp\" => \"2024-05-17T16:20:00.000Z\",\n  \"ecs.version\" => \"8.11.0\",\n  \"error.message\" => \"oops\",\n  \"error.stack_trace\" => \"** (RuntimeError) oops\n    test/logger_json/formatters/elastic_test.exs:421: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\"test logs caught errors\"/1\n    (logger_json 6.0.2) test/support/logger_case.ex:16: anonymous fn/1 in LoggerJSON.Case.capture_log/2\n    (ex_unit 1.16.3) lib/ex_unit/capture_io.ex:258: ExUnit.CaptureIO.do_with_io/3\n    (ex_unit 1.16.3) lib/ex_unit/capture_io.ex:134: ExUnit.CaptureIO.capture_io/2\n    (logger_json 6.0.2) test/support/logger_case.ex:15: LoggerJSON.Case.capture_log/2\n    test/logger_json/formatters/elastic_test.exs:419: LoggerJSON.Formatters.ElasticTest.\"test logs caught errors\"/1\n    (ex_unit 1.16.3) lib/ex_unit/runner.ex:472: ExUnit.Runner.exec_test/2\n    (stdlib 5.2.3) timer.erl:270: :timer.tc/2\n    (ex_unit 1.16.3) lib/ex_unit/runner.ex:394: anonymous fn/6 in ExUnit.Runner.spawn_test_monitor/4\n\",\n  \"error.type\" => \"Elixir.RuntimeError\",\n  \"log.level\" => \"error\",\n  \"log.logger\" => \"Elixir.LoggerJSON.Formatters.ElasticTest\",\n  \"log.origin\" => %{\n    \"file.line\" => 423,\n    \"file.name\" => \"/app/logger_json/test/logger_json/formatters/elastic_test.exs\",\n    \"function\" => \"test logs caught errors/1\"\n  },\n  \"message\" => \"Something went wrong\"\n}\n```","ref":"LoggerJSON.Formatters.Elastic.html#module-examples"},{"type":"module","title":"LoggerJSON.Formatters.GoogleCloud","doc":"Custom Erlang's [`:logger` formatter](https://www.erlang.org/doc/apps/kernel/logger_chapter.html#formatters) which\nwrites logs in a structured format that can be consumed by Google Cloud Logger.\n\nEven though the log messages on Google Cloud use [LogEntry](https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry)\nformat, not all the fields are available in the structured payload. The fields that are available can be found in the\n[special fields in structured payloads](https://cloud.google.com/logging/docs/agent/configuration#special_fields_in_structured_payloads).","ref":"LoggerJSON.Formatters.GoogleCloud.html"},{"type":"module","title":"Formatter Configuration - LoggerJSON.Formatters.GoogleCloud","doc":"The formatter can be configured with the following options:\n\n* `:project_id` (optional) - the Google Cloud project ID. This is required for correctly logging OpenTelemetry trace and\nspan IDs so that they can be linked to the correct trace in Google Cloud Trace. If it's not provided, the formatter will\ntry to read it from the environment variables `GOOGLE_CLOUD_PROJECT`, `GOOGLE_PROJECT_ID` or `GCLOUD_PROJECT`. Please keep\nin mind that environment will be read whenever you call the `new/1` function, so you should configure the formatter at runtime\n(in the `config/runtime.exs` or in your `application.ex`) if you rely on this feature.\n\n* `:service_context` (optional) - a map with the following keys:\n  * `:service` - the name of the service that is logging the message. Default: `node()`.\n  * `:version` - the version of the service that is logging the message.\n\n* `:reported_levels` (optional) - a list of log levels that should be reported as errors to Google Cloud Error Reporting.\nDefault: `[:emergency, :alert, :critical, :error]`.\n\nFor list of shared options see \"Shared options\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.GoogleCloud.html#module-formatter-configuration"},{"type":"module","title":"Metadata - LoggerJSON.Formatters.GoogleCloud","doc":"You can extend the log entry with some additional metadata:application\n\n  * `user_id`, `identity_id`, `actor_id`, `account_id` (ordered by precedence) - the ID of the user that is performing the action.\n  It will be included along with the error report for Google Cloud Error Reporting;\n\nFor list of other well-known metadata keys see \"Metadata\" in `LoggerJSON`.","ref":"LoggerJSON.Formatters.GoogleCloud.html#module-metadata"},{"type":"module","title":"Examples - LoggerJSON.Formatters.GoogleCloud","doc":"Regular message:\n\n    %{\n      \"logging.googleapis.com/operation\" => %{\"pid\" => \"#PID<0.228.0>\"},\n      \"logging.googleapis.com/sourceLocation\" => %{\n        \"file\" => \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n        \"function\" => \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs an LogEntry of a given level/1\",\n        \"line\" => 44\n      },\n      \"message\" => %{\"domain\" => [\"elixir\"], \"message\" => \"Hello\"},\n      \"severity\" => \"NOTICE\",\n      \"time\" => \"2024-04-11T21:32:46.957Z\"\n    }\n\nException message that will be recognized by Google Cloud Error Reporting:\n\n    %{\n      \"httpRequest\" => %{\n        \"protocol\" => \"HTTP/1.1\",\n        \"referer\" => \"http://www.example.com/\",\n        \"remoteIp\" => \"\",\n        \"requestMethod\" => \"PATCH\",\n        \"requestUrl\" => \"http://www.example.com/\",\n        \"status\" => 503,\n        \"userAgent\" => \"Mozilla/5.0\"\n      },\n      \"logging.googleapis.com/operation\" => %{\"pid\" => \"#PID<0.250.0>\"},\n      \"logging.googleapis.com/sourceLocation\" => %{\n        \"file\" => \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n        \"function\" => \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs exception http context/1\",\n        \"line\" => 301\n      },\n      \"@type\" => \"type.googleapis.com/google.devtools.clouderrorreporting.v1beta1.ReportedErrorEvent\",\n      \"context\" => %{\n        \"httpRequest\" => %{\n          \"protocol\" => \"HTTP/1.1\",\n          \"referer\" => \"http://www.example.com/\",\n          \"remoteIp\" => \"\",\n          \"requestMethod\" => \"PATCH\",\n          \"requestUrl\" => \"http://www.example.com/\",\n          \"status\" => 503,\n          \"userAgent\" => \"Mozilla/5.0\"\n        },\n        \"reportLocation\" => %{\n          \"filePath\" => \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n          \"functionName\" => \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs exception http context/1\",\n          \"lineNumber\" => 301\n        }\n      },\n      \"domain\" => [\"elixir\"],\n      \"message\" => \"Hello\",\n      \"serviceContext\" => %{\"service\" => \"nonode@nohost\"},\n      \"stack_trace\" => \"** (EXIT from #PID<0.250.0>) :foo\",\n      \"severity\" => \"DEBUG\",\n      \"time\" => \"2024-04-11T21:34:53.503Z\"\n    }","ref":"LoggerJSON.Formatters.GoogleCloud.html#module-examples"},{"type":"module","title":"LoggerJSON.Plug","doc":"A telemetry handler that logs request information in JSON format.\n\nPlease keep in mind that logging all requests may have a performance impact on your application,\nit's not recommended to use this module in high-throughput production environments.","ref":"LoggerJSON.Plug.html"},{"type":"function","title":"LoggerJSON.Plug.attach/3","doc":"Attaches the telemetry handler to the given event.","ref":"LoggerJSON.Plug.html#attach/3"},{"type":"function","title":"Available options - LoggerJSON.Plug.attach/3","doc":"* `:level` - log level which is used to log requests. Defaults to `:info`.","ref":"LoggerJSON.Plug.html#attach/3-available-options"},{"type":"function","title":"Dynamic log level - LoggerJSON.Plug.attach/3","doc":"In some cases you may wish to set the log level dynamically\non a per-query basis. To do so, set the `:level` option to\na tuple, `{Mod, Fun, Args}`. The query and map of time measures\nwill be prepended to the provided list of arguments.\n\nWhen invoked, your function must return a\n[`Logger.level()`](`t:Logger.level()/0`) or `false` to\ndisable logging for the request.","ref":"LoggerJSON.Plug.html#attach/3-dynamic-log-level"},{"type":"function","title":"Examples - LoggerJSON.Plug.attach/3","doc":"Attaching the telemetry handler to the `MyApp.Repo` events with the `:info` log level:\n\n    # in the endpoint\n    plug Plug.Telemetry, event_prefix: [:myapp, :plug]\n\n    # in your application.ex\n    LoggerJSON.Plug.attach(\"logger-json-requests\", [:myapp, :plug, :stop], :info)\n\nTo make plug broadcast those events see [`Plug.Telemetry`](https://hexdocs.pm/plug/Plug.Telemetry.html) documentation.\n\nYou can also attach to the `[:phoenix, :endpoint, :stop]` event to log request latency from Phoenix endpoints:\n\n    LoggerJSON.Plug.attach(\"logger-json-phoenix-requests\", [:phoenix, :endpoint, :stop], :info)","ref":"LoggerJSON.Plug.html#attach/3-examples"},{"type":"function","title":"LoggerJSON.Plug.telemetry_logging_handler/4","doc":"A telemetry handler that logs requests in a structured format.","ref":"LoggerJSON.Plug.html#telemetry_logging_handler/4"},{"type":"behaviour","title":"LoggerJSON.Redactor","doc":"This module provides a behaviour which allows to redact sensitive information from logs.\n\n*Note*: redactor will not be applied on `Jason.Fragment` structs if the encoder is `Jason`.\nFor more information about encoding and redacting see `LoggerJSON.Formatter.RedactorEncoder.encode/2`.","ref":"LoggerJSON.Redactor.html"},{"type":"callback","title":"LoggerJSON.Redactor.new/1","doc":"Initializes a new redactor configuration.","ref":"LoggerJSON.Redactor.html#c:new/1"},{"type":"callback","title":"Compile‑time vs. Runtime Configuration - LoggerJSON.Redactor.new/1","doc":"This function can’t be used in `config.exs` because that file is evaluated\nbefore your application modules are compiled and loaded, so `new/1` isn’t defined yet.\nYou can only call it in `config/runtime.exs` or from your application code.\n\nIf you must set up the redactor in `config.exs`, use the tuple format:\nthe first element is the module implementing `LoggerJSON.Redactor`,\nand the second is the options passed to `new/1`. For example:\n\n  config :logger, :default_handler,\n    formatter: {LoggerJSON.Formatters.Basic, redactors: [\n      {MyRedactor, [option1: :value1]}\n    ]}\n\nNote that tuple‑based configs are resolved for each log entry,\nwhich can increase logging overhead.","ref":"LoggerJSON.Redactor.html#c:new/1-compile-time-vs-runtime-configuration"},{"type":"callback","title":"LoggerJSON.Redactor.redact/3","doc":"Takes a key and a value and returns a redacted value.\n\nThis callback will be applied on key-value pairs, like elements of structs, maps or keyword lists.","ref":"LoggerJSON.Redactor.html#c:redact/3"},{"type":"module","title":"LoggerJSON.Redactors.RedactKeys","doc":"A simple redactor which replace the value of the keys with `\"[REDACTED]\"`.\n\nIt takes list of keys to redact as an argument, eg.:\n```elixir\nconfig :logger, :default_handler,\n  formatter:\n    LoggerJSON.Formatters.Basic.new(\n      redactors: [\n        LoggerJSON.Redactors.RedactKeys.new([\"password\"])\n      ]\n    )\n```\n\nKeep in mind that the key will be converted to binary before sending it to the redactor.","ref":"LoggerJSON.Redactors.RedactKeys.html"},{"type":"extras","title":"License","doc":"The MIT License (MIT)\n\nCopyright (c) 2016 Andrii Dryga\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# LoggerJSON\n\n[![Build Status](https://github.com/Nebo15/logger_json/actions/workflows/ci.yml/badge.svg)](https://github.com/Nebo15/logger_json/actions/workflows/ci.yml)\n[![Coverage Status](https://coveralls.io/repos/github/Nebo15/logger_json/badge.svg?branch=master)](https://coveralls.io/github/Nebo15/logger_json?branch=master)\n[![Module Version](https://img.shields.io/hexpm/v/logger_json.svg)](https://hex.pm/packages/logger_json)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/logger_json/)\n[![Hex Download Total](https://img.shields.io/hexpm/dt/logger_json.svg)](https://hex.pm/packages/logger_json)\n[![License](https://img.shields.io/hexpm/l/logger_json.svg)](https://github.com/Nebo15/logger_json/blob/master/LICENSE.md)\n\nA collection of formatters and utilities for JSON-based logging for various cloud tools and platforms.","ref":"readme.html"},{"type":"extras","title":"Supported formatters - Overview","doc":"- [`LoggerJSON.Formatters.Basic`](https://hexdocs.pm/logger_json/LoggerJSON.Formatters.Basic.html) - a basic JSON formatter that logs messages in a structured, but generic format, can be used with any JSON-based logging system.\n\n- [`LoggerJSON.Formatters.GoogleCloud`](https://hexdocs.pm/logger_json/LoggerJSON.Formatters.GoogleCloud.html) - a formatter that logs messages in a structured format that can be consumed by Google Cloud Logger and Google Cloud Error Reporter.\n\n- [`LoggerJSON.Formatters.Datadog`](https://hexdocs.pm/logger_json/LoggerJSON.Formatters.Datadog.html) - a formatter that logs messages in a structured format that can be consumed by Datadog.\n\n- [`LoggerJSON.Formatters.Elastic`](https://hexdocs.pm/logger_json/LoggerJSON.Formatters.Elastic.html) - a formatter that logs messages in a structured format that conforms to the [Elastic Common Schema (ECS)](https://www.elastic.co/guide/en/ecs/8.11/ecs-reference.html), so it can be consumed by ElasticSearch, LogStash, FileBeat and Kibana.","ref":"readme.html#supported-formatters"},{"type":"extras","title":"Installation - Overview","doc":"Add `logger_json` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # ...\n    {:logger_json, \"~> 7.0\"}\n    # ...\n  ]\nend\n```\n\nand install it running `mix deps.get`.\n\nThen, enable the formatter in your `runtime.exs`:\n\n```elixir\nconfig :logger, :default_handler,\n  formatter: LoggerJSON.Formatters.Basic.new(metadata: [:request_id])\n```\n\nor inside your application code (eg. in your `application.ex`):\n\n```elixir\nformatter = LoggerJSON.Formatters.Basic.new(metadata: :all)\n:logger.update_handler_config(:default, :formatter, formatter)\n```\n\nor inside your `config.exs` (notice that `new/1` is not available here\nand tuple format must be used):\n\n```elixir\nconfig :logger, :default_handler,\n  formatter: {LoggerJSON.Formatters.Basic, metadata: [:request_id]}\n```\n\nYou might also want to format the log messages when migrations are running:\n\n```elixir\nconfig :domain, MyApp.Repo,\n  # ...\n  start_apps_before_migration: [:logger_json]\n```\n\nAnd you might want to make logging level configurable using an `LOG_LEVEL` environment variable (in `application.ex`):\n\n```elixir\nLoggerJSON.configure_log_level_from_env!()\n```\n\nAdditionally, you may also be try [redirecting otp reports to Logger](https://hexdocs.pm/logger/Logger.html#module-configuration) (see \"Configuration\" section).","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Overview","doc":"Configuration can be set using 2nd element of the tuple of the `:formatter` option in `Logger` configuration.\nFor example in `config.exs`:\n\n```elixir\nconfig :logger, :default_handler,\n  formatter: LoggerJSON.Formatters.GoogleCloud.new(metadata: :all, project_id: \"logger-101\")\n```\n\nor during runtime:\n\n```elixir\nformatter = LoggerJSON.Formatters.Basic.new(%{metadata: {:all_except, [:conn]}})\n:logger.update_handler_config(:default, :formatter, formatter)\n```\n\nBy default, `LoggerJSON` is using `Jason` as the JSON encoder. If you use Elixir 1.18 or later, you can\nuse the built-in `JSON` module as the encoder. To do this, you need to set the `:encoder` option in your\n`config.exs` file. This setting is only available at compile-time:\n\n    config :logger_json, encoder: JSON","ref":"readme.html#configuration"},{"type":"extras","title":"Docs - Overview","doc":"The docs can be found at [https://hexdocs.pm/logger_json](https://hexdocs.pm/logger_json).","ref":"readme.html#docs"},{"type":"extras","title":"Examples - Overview","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"Basic - Overview","doc":"```json\n{\n  \"message\": \"Hello\",\n  \"metadata\": {\n    \"domain\": [\"elixir\"]\n  },\n  \"severity\": \"notice\",\n  \"time\": \"2024-04-11T21:31:01.403Z\"\n}\n```","ref":"readme.html#basic"},{"type":"extras","title":"Google Cloud Logger - Overview","doc":"Follows the [Google Cloud Logger LogEntry](https://cloud.google.com/logging/docs/reference/v2/rest/v2/LogEntry) format,\nfor more details see [special fields in structured payloads](https://cloud.google.com/logging/docs/agent/configuration#special_fields_in_structured_payloads).\n\n```json\n{\n  \"logging.googleapis.com/trace\": \"projects/my-projectid/traces/0679686673a\",\n  \"logging.googleapis.com/spanId\": \"000000000000004a\",\n  \"logging.googleapis.com/operation\": {\n    \"pid\": \"#PID<0.29081.0>\"\n  },\n  \"logging.googleapis.com/sourceLocation\": {\n    \"file\": \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n    \"function\": \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs an LogEntry of a given level/1\",\n    \"line\": 44\n  },\n  \"message\": {\n    \"domain\": [\"elixir\"],\n    \"message\": \"Hello\"\n  },\n  \"severity\": \"NOTICE\",\n  \"time\": \"2024-04-12T15:07:55.020Z\"\n}\n```\n\nand this is how it looks in Google Cloud Logger:\n\n```json\n{\n  \"insertId\": \"1d4hmnafsj7vy1\",\n  \"jsonPayload\": {\n    \"message\": \"Hello\",\n    \"logging.googleapis.com/spanId\": \"000000000000004a\",\n    \"domain\": [\"elixir\"],\n    \"time\": \"2024-04-12T15:07:55.020Z\"\n  },\n  \"resource\": {\n    \"type\": \"gce_instance\",\n    \"labels\": {\n      \"zone\": \"us-east1-d\",\n      \"project_id\": \"firezone-staging\",\n      \"instance_id\": \"3168853301020468373\"\n    }\n  },\n  \"timestamp\": \"2024-04-12T15:07:55.023307594Z\",\n  \"severity\": \"NOTICE\",\n  \"logName\": \"projects/firezone-staging/logs/cos_containers\",\n  \"operation\": {\n    \"id\": \"F8WQ1FsdFAm5ZY0AC1PB\",\n    \"producer\": \"#PID<0.29081.0>\"\n  },\n  \"trace\": \"projects/firezone-staging/traces/bc007e40a2e9edffa23785d8badc43b8\",\n  \"sourceLocation\": {\n    \"file\": \"lib/phoenix/logger.ex\",\n    \"line\": \"231\",\n    \"function\": \"Elixir.Phoenix.Logger.phoenix_endpoint_stop/4\"\n  },\n  \"receiveTimestamp\": \"2024-04-12T15:07:55.678986520Z\"\n}\n```\n\nException that can be sent to Google Cloud Error Reporter:\n\n```json\n{\n  \"httpRequest\": {\n    \"protocol\": \"HTTP/1.1\",\n    \"referer\": \"http://www.example.com/\",\n    \"remoteIp\": \"\",\n    \"requestMethod\": \"PATCH\",\n    \"requestUrl\": \"http://www.example.com/\",\n    \"status\": 503,\n    \"userAgent\": \"Mozilla/5.0\"\n  },\n  \"logging.googleapis.com/operation\": {\n    \"pid\": \"#PID<0.250.0>\"\n  },\n  \"logging.googleapis.com/sourceLocation\": {\n    \"file\": \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n    \"function\": \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs exception http context/1\",\n    \"line\": 301\n  },\n  \"@type\": \"type.googleapis.com/google.devtools.clouderrorreporting.v1beta1.ReportedErrorEvent\",\n  \"context\": {\n    \"httpRequest\": {\n      \"protocol\": \"HTTP/1.1\",\n      \"referer\": \"http://www.example.com/\",\n      \"remoteIp\": \"\",\n      \"requestMethod\": \"PATCH\",\n      \"requestUrl\": \"http://www.example.com/\",\n      \"status\": 503,\n      \"userAgent\": \"Mozilla/5.0\"\n    },\n    \"reportLocation\": {\n      \"filePath\": \"/Users/andrew/Projects/os/logger_json/test/formatters/google_cloud_test.exs\",\n      \"functionName\": \"Elixir.LoggerJSON.Formatters.GoogleCloudTest.test logs exception http context/1\",\n      \"lineNumber\": 301\n    }\n  },\n  \"domain\": [\"elixir\"],\n  \"message\": \"Hello\",\n  \"serviceContext\": {\n    \"service\": \"nonode@nohost\"\n  },\n  \"stack_trace\": \"** (EXIT from #PID<0.250.0>) :foo\",\n  \"severity\": \"DEBUG\",\n  \"time\": \"2024-04-11T21:34:53.503Z\"\n}\n```","ref":"readme.html#google-cloud-logger"},{"type":"extras","title":"Datadog - Overview","doc":"Adheres to the [default standard attribute list](https://docs.datadoghq.com/logs/processing/attributes_naming_convention/#default-standard-attribute-list)\nas much as possible.\n\n```json\n{\n  \"domain\": [\"elixir\"],\n  \"http\": {\n    \"method\": \"GET\",\n    \"referer\": \"http://www.example2.com/\",\n    \"request_id\": null,\n    \"status_code\": 200,\n    \"url\": \"http://www.example.com/\",\n    \"url_details\": {\n      \"host\": \"www.example.com\",\n      \"path\": \"/\",\n      \"port\": 80,\n      \"queryString\": \"\",\n      \"scheme\": \"http\"\n    },\n    \"useragent\": \"Mozilla/5.0\"\n  },\n  \"logger\": {\n    \"file_name\": \"/Users/andrew/Projects/os/logger_json/test/formatters/datadog_test.exs\",\n    \"line\": 239,\n    \"method_name\": \"Elixir.LoggerJSON.Formatters.DatadogTest.test logs http context/1\",\n    \"thread_name\": \"#PID<0.225.0>\"\n  },\n  \"message\": \"Hello\",\n  \"network\": {\n    \"client\": {\n      \"ip\": \"127.0.0.1\"\n    }\n  },\n  \"syslog\": {\n    \"hostname\": \"MacBook-Pro\",\n    \"severity\": \"debug\",\n    \"timestamp\": \"2024-04-11T23:10:47.967Z\"\n  }\n}\n```","ref":"readme.html#datadog"},{"type":"extras","title":"Elastic - Overview","doc":"Follows the [Elastic Common Schema (ECS)](https://www.elastic.co/guide/en/ecs/8.11/ecs-reference.html) format.\n\n```json\n{\n  \"@timestamp\": \"2024-05-21T15:17:35.374Z\",\n  \"ecs.version\": \"8.11.0\",\n  \"log.level\": \"info\",\n  \"log.logger\": \"Elixir.LoggerJSON.Formatters.ElasticTest\",\n  \"log.origin\": {\n    \"file.line\": 18,\n    \"file.name\": \"/app/logger_json/test/logger_json/formatters/elastic_test.exs\",\n    \"function\": \"test logs message of every level/1\"\n  },\n  \"message\": \"Hello\"\n}\n```\n\nWhen an error is thrown, the message field is populated with the error message and the `error.` fields will be set:\n\n> Note: when throwing a custom exception type that defines the fields `id` and/or `code`, then the `error.id` and/or `error.code` fields will be set respectively.\n\n```json\n{\n  \"@timestamp\": \"2024-05-21T15:20:11.623Z\",\n  \"ecs.version\": \"8.11.0\",\n  \"error.message\": \"runtime error\",\n  \"error.stack_trace\": \"** (RuntimeError) runtime error\\n    test/logger_json/formatters/elastic_test.exs:191: anonymous fn/0 in LoggerJSON.Formatters.ElasticTest.\\\"test logs exceptions\\\"/1\\n\",\n  \"error.type\": \"Elixir.RuntimeError\",\n  \"log.level\": \"error\",\n  \"message\": \"runtime error\"\n}\n```\n\nAny custom metadata fields will be added to the root of the message, so that your application can fill any other ECS fields that you require:\n\n> Note that this also allows you to produce messages that do not strictly adhere to the ECS specification.\n\n```json\n// Logger.info(\"Hello\") with Logger.metadata(:\"device.model.name\": \"My Awesome Device\")\n// or Logger.info(\"Hello\", \"device.model.name\": \"My Awesome Device\")\n{\n  \"@timestamp\": \"2024-05-21T15:17:35.374Z\",\n  \"ecs.version\": \"8.11.0\",\n  \"log.level\": \"info\",\n  \"log.logger\": \"Elixir.LoggerJSON.Formatters.ElasticTest\",\n  \"log.origin\": {\n    \"file.line\": 18,\n    \"file.name\": \"/app/logger_json/test/logger_json/formatters/elastic_test.exs\",\n    \"function\": \"test logs message of every level/1\"\n  },\n  \"message\": \"Hello\",\n  \"device.model.name\": \"My Awesome Device\"\n}\n```","ref":"readme.html#elastic"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2016 Andrew Dryga\n\nReleased under the MIT License, which can be found in [LICENSE.md](./LICENSE.md).","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}